{*
 * ARCLIBTD.PAS
 *
 *  Source file for ArchiveLib 2.0
 *
 *  Copyright (c) Greenleaf Software, Inc. 1994-1996
 *  All Rights Reserved
 *
 * CONTENTS
 *
 *  Delphi declarations, plus the following functions:
 *
 *  newALPkArchive()
 *  ALArchiveGetComment()
 *  ALEntryGetComment()
 *  ALArchiveSetComment()
 *  ALStorageGetName()
 *  ALReadDir()
 *  ALFreeDir()
 *  ALWriteDir()
 *  ALZipDirEntry.get_name
 *  ALZipDirEntry.get_comment
 *  ALZipDirEntry.put_name
 *  ALZipDirEntry.put_comment
 *  ALZipDirEntry.Create
 *  ALZipDirEntry.Destroy
 *  ALCreate()
 *  ALAppend()
 *  ALDelete()
 *  ALExtract()
 *
 * REVISION HISTORY
 *
 *   March 21, 1996  2.01A : New release
 *}

unit ArcLib;

interface

uses WinTypes, SysUtils;

const
  ALLibName = 'TD20FVB.DLL';

  AL_MONITOR_OBJECTS = 0;
  AL_MONITOR_JOB = 1;

  AL_SEND_BYTE_COUNT = 0;
  AL_SEND_RATIO = 1;

  AL_CANT_OPEN_BUFFER = -1200;
  AL_CANT_ALLOCATE_MEMORY = -1199;
  AL_CANT_CREATE_ENGINE = -1198;
  AL_CANT_CREATE_STORAGE_OBJECT = -1197;
  AL_RENAME_ERROR = -1196;
  AL_CANT_OPEN_FILE = -1195;
  AL_SEEK_ERROR = -1194;
  AL_READ_ERROR = -1193;
  AL_WRITE_ERROR = -1192;
  AL_DELETE_ERROR = -1191;
  AL_ILLEGAL_PARAMETER = -1190;
  AL_INTERNAL_ERROR = -1189;
  AL_USER_ABORT = -1188;
  AL_SERVER_NOT_PRESENT = -1187;
  AL_COMPRESSION_TYPE_MISMATCH = -1186;
  AL_NEED_LENGTH = -1185;
  AL_CRC_ERROR = -1184;
  AL_COMPARE_ERROR = -1183;
  AL_UNKNOWN_COMPRESSION_TYPE = -1182;
  AL_UNKNOWN_STORAGE_OBJECT = -1181;
  AL_INVALID_ARCHIVE = -1180;
  AL_LOGIC_ERROR = -1179;
  AL_BACKUP_FAILURE = -1178;
  AL_GETSEL_ERROR = -1177;
  AL_DUPLICATE_ENTRY = -1176;
  AL_END_OF_FILE = -1;
  AL_SUCCESS = 0;

  AL_DEFAULT = -2;

  AL_TRAVERSE = 1;
  AL_DONT_TRAVERSE = 0;

  AL_UPPER = 0;
  AL_LOWER = 1;
  AL_MIXED = 2;

  AL_GREENLEAF_COPY = -1;
  AL_GREENLEAF_LEVEL_0 = 0;
  AL_GREENLEAF_LEVEL_1 = 1;
  AL_GREENLEAF_LEVEL_2 = 2;
  AL_GREENLEAF_LEVEL_3 = 3;
  AL_GREENLEAF_LEVEL_4 = 4;

  AL_COMPRESSION_COPY = 0;
  AL_COMPRESSION_GREENLEAF = 1;
  AL_COMPRESSION_DEFLATE = 108;

type
    hALCompressor = ^Char;
    hALDecompressor = ^Char;
    hALEntryList = ^Char;
    hALEntry = ^Char;
    hALMonitor = ^Char;
    hALToolKit = ^Char;
    hALArchive = ^Char;
    hALStorage = ^Char;
    hALExpander = ^Char;

{*
 * ALZipDir is the data structure used to hold an archive
 * directory for the simple interface functions.
 *}

ALZipDirEntry = class
public
    constructor Create;
    destructor Destroy;
public
    pname               : PChar;
    pcomment            : PChar;
    compressed_size     : LongInt;
    compressed_position : LongInt;
    size                : LongInt;
    crc                 : LongInt;
    mark                : SmallInt;
    month               : SmallInt;
    date                : SmallInt;
    year                : SmallInt;
    hour                : SmallInt;
    minute              : SmallInt;
    second              : SmallInt;
    r                   : boolean ;
    a                   : boolean;
    s                   : boolean;
    h                   : boolean;
    d                   : boolean;
    level               : byte;
    function get_name : string;
    procedure put_name( x : string );
    property name : string read get_name write put_name;
    function get_comment : string;
    procedure put_comment( x : string );
    property comment : string read get_comment write put_comment;
end;

const
  AL_MAX_ZIP_DIR = ( 65520 div Sizeof( ALZipDirEntry ) ) - 1;

type

ALZipDirArray = array[ 0..AL_MAX_ZIP_DIR ] of ALZipDirEntry;
ALZipDir = ^ALZipDirArray;

{ The simple interface }
function ALReadDir( filename : string;
                    var count : integer;
                    var error : integer ) : ALZipDir;
procedure ALFreeDir( z : ALZipDir ); stdcall;
function ALWriteDir( z : ALZipDir ) : integer; stdcall;
procedure ALWriteDirEntryDelphi( z : ALZipDirEntry;
                                 list : hALEntryList ); stdcall;
function ALCreatePChar( archive_name : PChar;
                        input_files : PChar;
                        strip_path : integer;
                        text_window : Hwnd;
                        file_progress_window : Hwnd;
                        job_progress_window : Hwnd ) : integer; stdcall;
function ALCreate( archive_name : String;
                   input_files : String;
                   strip_path : integer;
                   text_window : Hwnd;
                   file_progress_window : Hwnd;
                   job_progress_window : Hwnd ) : integer; stdcall;
function ALAppendPChar( archive_name : PChar;
                        input_files : PChar;
                        strip_path : integer;
                        text_window : Hwnd;
                        file_progress_window : Hwnd;
                        job_progress_window : Hwnd ) : integer; stdcall;
function ALAppend( archive_name : String;
                   input_files : String;
                   strip_path : integer;
                   text_window : Hwnd;
                   file_progress_window : Hwnd;
                   job_progress_window : Hwnd ) : integer; stdcall;
function ALDelete( z : ALZipDir;
                   text_window : Hwnd;
                   file_progress_window : Hwnd;
                   job_progress_window : Hwnd ) : integer; stdcall;
function ALExtract( z : ALZipDir;
                    strip_path : integer;
                    text_window : Hwnd;
                    file_progress_window : Hwnd;
                    job_progress_window : Hwnd ) : integer; stdcall;
function newALSimpleMonitor( text_window : Hwnd;
                             file_progress_window : Hwnd;
                             job_progress_window : Hwnd ) : hALMonitor; stdcall;
procedure ALArchiveSetStripOnInsert( this_object : hALArchive;
                                     flag : integer ); stdcall;
procedure ALArchiveSetStripOnExtract( this_object : hALArchive;
                                      flag : integer ); stdcall;
{*
 * ALCompressor declaration
 *}
function ALPkDecompressorLevel( this_object : hALDecompressor ) : SmallInt; stdcall;
function ALDecompressorGetTypeCode( this_object : hALDecompressor ) : Integer; stdcall;
function newALPkCompressor( level : Integer;
                            window_bits : Integer;
                            mem_level : Integer ) : hALCompressor; stdcall;

{*
 * ALEntryList declaration
 *}
function newALListPkCompressFileTools( monitor : hALMonitor;
                                       level : Integer;
                                       window_bits : Integer;
                                       mem_level : Integer ) : hALEntryList; stdcall;
function newALListPkDecompressFileTools( monitor : hALMonitor ) : hALEntryList; stdcall;
procedure deleteALEntryList( this_object : hALEntryList ); stdcall;
function ALEntryListGetFirstEntry( this_object : hALEntryList ) : hALEntry; stdcall;

{*
 * ALArchive declaration
 *}

function newALPkArchivePChar( name : PChar ) : hALArchive; stdcall;
function newALPkArchive( name : String ) : hALArchive;
function ALArchiveGetCommentPChar( this_object : hALArchive ) : PChar; stdcall;
function ALArchiveGetComment( this_object : hALArchive ) : String; stdcall;
function ALArchiveGetStorage( this_object : hALArchive ) : hALStorage; stdcall;
procedure deleteALArchive( this_object : hALArchive ); stdcall;
function ALArchiveDelete( this_object : hALArchive;
                          list : hALEntryList;
                          output_archive : hALArchive ) : Integer; stdcall;
function ALArchiveExtract( this_object : hALArchive;
                           list : hALEntryList ) : Integer; stdcall;
function ALArchiveSetCommentPChar( this_object : hALArchive;
                                 comment : PChar ) : Integer; stdcall;
function ALArchiveSetComment( this_object : hALArchive;
                            comment : String ) : Integer; stdcall;
function ALArchiveWriteDirectory( this_object : hALArchive;
                                  list : hALEntryList ) : Integer; stdcall;
function ALArchiveReadDirectory( this_object : hALArchive;
                                 list : hALEntryList ) : Integer; stdcall;

{*
 * ALEntry declaration
 *}
function ALEntryGetCompressedSize( this_object : hALEntry ) : LongInt; stdcall;
function ALEntryGetCompressedPosition( this_object : hALEntry ) : LongInt; stdcall;
function ALEntryGetCrc32( this_object : hALEntry ) : LongInt; stdcall;
function ALEntryGetCommentPChar( this_object : hALEntry ) : PChar; stdcall;
function ALEntryGetComment( this_object : hALEntry ) : String; stdcall;
function ALEntryGetMark( this_object : hALEntry ) : Integer; stdcall;
procedure deleteALEntry( this_object : hALEntry ); stdcall;
function ALEntryGetNextEntry( this_object : hALEntry ) : hALEntry; stdcall;
function ALEntryGetStorage( this_object : hALEntry ) : hALStorage; stdcall;
function ALEntryGetDecompressor( this_object : hALEntry ) : hALDecompressor; stdcall;

{*
 * ALStorage declaration
 *}
function ALStorageTell( this_object : hALStorage ) : LongInt; stdcall;
function ALStorageReadOnly( storage : hALStorage ) : Integer; stdcall;
function ALStorageSystem( storage : hALStorage ) : Integer; stdcall;
function ALStorageHidden( storage : hALStorage ) : Integer; stdcall;
function ALStorageArchive( storage : hALStorage ) : Integer; stdcall;
function ALStorageDirectory( storage : hALStorage ) : Integer; stdcall;
function ALStorageGetSize( this_object : hALStorage ) : LongInt; stdcall;
procedure ALStorageInitCrc32( this_object : hALStorage; seed : LongInt ); stdcall;
function ALStorageGetNamePChar( this_object : hALStorage ) : PChar; stdcall;
function ALStorageGetName( this_object : hALStorage ) : String; stdcall;

{ 2 functions that didn't make it into the manual }

procedure ALStorageGetTime( this_object : hALStorage;
                            var hour : SmallInt;
                            var minute : SmallInt;
                            var second : SmallInt); stdcall;
procedure ALStorageGetDate( this_object : hALStorage;
                            var month : SmallInt;
                            var date : SmallInt;
                            var year : SmallInt ); stdcall;

{*
 * ALMonitor declaration
 *}
procedure deleteALMonitor( this_object : hALMonitor ); stdcall;

implementation

{*
 * Simple interface implementation
 *}
function ALCreatePChar; external ALLibName name 'ALCreateVB';
function ALAppendPChar; external ALLibName name 'ALAppendVB';
function newALSimpleMonitor; external ALLibName name 'newALSimpleMonitor';
procedure ALArchiveSetStripOnInsert; external ALLibName name 'ALArchiveStripOnInsert';
procedure ALArchiveSetStripOnExtract; external ALLibName name 'ALArchiveSetStripOnExtract';
procedure ALWriteDirEntryDelphi; external ALLibName name 'ALWriteDirEntryDelphi';

{*
 * ALCompressor implementation
 *}
function ALPkDecompressorLevel; external ALLibName name 'ALPkDecompressorLevel';
function ALDecompressorGetTypeCode; external ALLibName name 'ALDecompressorGetTypeCode';
function newALPkCompressor; external ALLibName name 'newALPkCompressor';

{*
 * ALEntryList implementation
 *}
function newALListPkCompressFileTools; external ALLibName name 'newALListPkCompressFileTools';
function newALListPkDecompressFileTools; external ALLibName name 'newALListPkDecompressFileTools';
procedure deleteALEntryList; external ALLibName name 'deleteALEntryList';
function ALEntryListGetFirstEntry; external ALLibName name 'ALEntryListGetFirstEntry';

{*
 * ALArchive implementation
 *}

function newALPkArchivePChar; external ALLibName name 'newALPkArchive';
function ALArchiveGetCommentPChar; external ALLibName name 'ALArchiveGetComment';
function ALArchiveGetStorage; external ALLibName name 'ALArchiveGetStorage';
procedure deleteALArchive; external ALLibName name 'deleteALArchive';
function ALArchiveDelete; external ALLibName name 'ALArchiveDelete';
function ALArchiveExtract; external ALLibName name 'ALArchiveExtract';
function ALArchiveSetCommentPChar; external ALLibName name 'ALArchiveSetComment';
function ALArchiveWriteDirectory; external ALLibName name 'ALArchiveWriteDirectory';
function ALArchiveReadDirectory; external ALLibName name 'ALArchiveReadDirectory';

{*
 * ALEntry implementation
 *}

function ALEntryGetCompressedSize; external ALLibName name 'ALEntryGetCompressedSize';
function ALEntryGetCompressedPosition; external ALLibName name 'ALEntryGetCompressedPosition';
function ALEntryGetCrc32; external ALLibName name 'ALEntryGetCrc32';
function ALEntryGetCommentPChar; external ALLibName name 'ALEntryGetComment';
function ALEntryGetMark; external ALLibName name 'ALEntryGetMark';
procedure deleteALEntry; external ALLibName name 'deleteALEntry';
function ALEntryGetNextEntry; external ALLibName name 'ALEntryGetNextEntry';
function ALEntryGetStorage; external ALLibName name 'ALEntryGetStorage';
function ALEntryGetDecompressor; external ALLibName name 'ALEntryGetDecompressor';

{*
 * ALStorage implementation
 *}
function ALStorageTell; external ALLibName name 'ALStorageTell';
function ALStorageReadOnly; external ALLibName name 'ALStorageReadOnly';
function ALStorageSystem; external ALLibName name 'ALStorageSystem';
function ALStorageHidden; external ALLibName name 'ALStorageHidden';
function ALStorageArchive; external ALLibName name 'ALStorageArchive';
function ALStorageDirectory; external ALLibName name 'ALStorageDirectory';
function ALStorageGetSize; external ALLibName name 'ALStorageGetSize';
procedure ALStorageInitCrc32; external ALLibName name 'ALStorageInitCrc32';
function ALStorageGetNamePChar; external ALLibName name 'ALStorageGetName';
procedure ALStorageGetTime; external ALLibName name 'ALStorageGetTime';
procedure ALStorageGetDate; external ALLibName name 'ALStorageGetDate';

{*
 * ALMonitor implementation
 *}
procedure deleteALMonitor; external ALLibName name 'deleteALMonitor';

{*
 * DELPHI WRAPPER FUNCTION
 *
 * NAME
 *
 *  newALPkArchive()
 *
 *  This wrapper function simply performs translation between C-Style
 *  PChar data and Pascal-style Strings.  For complete documentation
 *  on what this function does, see the source for the C/VB function
 *  of the same name in PKARCF.CPP
 *
 * REVISION HISTORY
 *
 *  February 14, 1996  2.0A : First release
 *}

function newALPkArchive( name : String ) : hALArchive;
var
  p : PChar;
begin
  p := StrAlloc( Length( name ) + 1 );
  StrPCopy( p, name );
  newALPkArchive := newALPkArchivePChar( p );
  StrDispose( p );
end;

{*
 * DELPHI WRAPPER FUNCTION
 *
 * NAME
 *
 *  ALArchiveGetComment()
 *
 *  This wrapper function simply performs translation between C-Style
 *  PChar data and Pascal-style Strings.  For complete documentation
 *  on what this function does, see the source for the C/VB function
 *  of the same name in CXL_ARC.CPP
 *
 * REVISION HISTORY
 *
 *  February 14, 1996  2.0A : First release
 *}

function ALArchiveGetComment( this_object : hALArchive ) : String;
var
  p : PChar;
begin
  p := ALArchiveGetCommentPChar( this_object );
  if p = nil Then
    ALArchiveGetComment := ''
  else
    ALArchiveGetComment := StrPas( p );
end;

{*
 * DELPHI WRAPPER FUNCTION
 *
 * NAME
 *
 *  ALEntryGetComment()
 *
 *  This wrapper function simply performs translation between C-Style
 *  PChar data and Pascal-style Strings.  For complete documentation
 *  on what this function does, see the source for the C/VB function
 *  of the same name in CXL_ENTR.CPP
 *
 * REVISION HISTORY
 *
 *  February 14, 1996  2.0A : First release
 *}

function ALEntryGetComment( this_object : hALEntry ) : String;
var
  p : PChar;
begin
  p := ALEntryGetCommentPChar( this_object );
  if p = nil Then
    ALEntryGetComment := ''
  else
    ALEntryGetComment := StrPas( p );
end;

{*
 * DELPHI WRAPPER FUNCTION
 *
 * NAME
 *
 *  ALArchiveSetComment()
 *
 *  This wrapper function simply performs translation between C-Style
 *  PChar data and Pascal-style Strings.  For complete documentation
 *  on what this function does, see the source for the C/VB function
 *  of the same name in ARCSC.CPP
 *
 * REVISION HISTORY
 *
 *  February 14, 1996  2.0A : First release
 *}

function ALArchiveSetComment( this_object : hALArchive;
                              comment : String ) : Integer;
var
  p : PChar;
begin
  p := StrAlloc( Length( comment ) + 1 );
  StrPCopy( p, comment );
  ALArchiveSetComment := ALArchiveSetCommentPChar( this_object, p );
  StrDispose( p );
end;

{*
 * DELPHI WRAPPER FUNCTION
 *
 * NAME
 *
 *  ALStorageGetName()
 *
 *  This wrapper function simply performs translation between C-Style
 *  PChar data and Pascal-style Strings.  For complete documentation
 *  on what this function does, see the source for the C/VB function
 *  of the same name in CXL_STOR.CPP
 *
 * REVISION HISTORY
 *
 *  February 14, 1996  2.0A : First release
 *}

function ALStorageGetName( this_object : hALStorage ) : String;
var
  p : PChar;
begin
  p := ALStorageGetNamePChar( this_object );
  if p = nil Then
    ALStorageGetName := ''
  else
    ALStorageGetName := StrPas( p );
end;

{*
 * NAME
 *
 *  ALReadDir()
 *
 * PLATFORMS/ENVIRONMENTS
 *
 *  Windows
 *  Delphi
 *
 * SHORT DESCRIPTION
 *
 *  The simplified interface function reads in the directory of a ZIP file.
 *
 * DELPHI SYNOPSIS
 *
 *  function ALReadDir( filename : string;
 *                      var count : integer;
 *                      var error : integer ) : ALZipDir;
 *
 * ARGUMENTS
 *
 *  filename      :  The name of the zip archive whose directory
 *                   you want to read.
 *
 *  count         :  The integer that is going to receive the count of
 *                   items in the directory.
 *
 *  error         :  The integer that is going to receive the status from
 *                   the ALReadDir operation.
 *
 * DESCRIPTION
 *
 *  This function reads in the directory information from a ZIP
 *  file, then takes each entry and inserts it into an ALZipDir array.
 *  This means it has to take each record and reformat the data so
 *  that it fits in this fixed array.  Formatting the array is
 *  an inelegant brute force operation where each element is either
 *  calculated or retrieved using an ArchiveLiv C interface call.
 *
 * RETURNS
 *
 *  The pointer to an ALZipDir array.  If a memory allocation failure
 *  occurs, you just get a nil back.  This function also returns the
 *  count and error information as described above.
 *
 * EXAMPLE
 *
 * SEE ALSO
 *
 * REVISION HISTORY
 *
 *   February 14, 1996  2.0A : New Release
 *
 *   March 26, 1996     2.01A : Added code to fix an error when reading
 *                              incompatible PKZip 1.x archives.  When
 *                              this happens, ALEntryGetCompressor() might
 *                              return nil.
 *}

 function ALReadDir( filename : string;
                    var count : integer;
                    var error : integer ) : ALZipDir;
var
  arc : hALArchive;
  d : hALDecompressor;
  list : hALEntryList;
  i : integer;
  entry : hALEntry;
  z : ALZipDir;
  storage : hALStorage;
begin
  ALReadDir := nil;
  arc := newALPkArchive( filename );
  list := newALListPkDecompressFileTools( nil );
  error := ALArchiveReadDirectory( arc, list );
  count := -1;
  if (arc <> nil) and (list <> nil) and ( error = AL_SUCCESS ) then
  begin
    count := 0;
    entry := ALEntryListGetFirstEntry( list );
    while entry <> nil do
    begin
      count := count + 1;
      entry := ALEntryGetNextEntry( entry );
    end;
  end;
  if (count < (AL_MAX_ZIP_DIR - 2 )) and (count > -1 ) then
  begin
    GetMem( z, sizeof( ALZipDirEntry ) * ( count + 1 ) );
    i := 0;
    entry := ALEntryListGetFirstEntry( list );
    for i := 0 to count - 1 do
    begin
      storage := ALEntryGetStorage( entry );
      z^[ i ] := ALZipDirEntry.Create;
      z^[ i ].name := ALStorageGetName( storage );
      z^[ i ].comment := ALEntryGetComment( entry );
      z^[ i ].compressed_size := ALEntryGetCompressedSize( entry );
      z^[ i ].compressed_position := ALEntryGetCompressedPosition( entry );
      z^[ i ].size := ALStorageGetSize( storage );
      z^[ i ].crc := ALEntryGetCrc32( entry );
      z^[ i ].mark := ALEntryGetMark( entry );
      z^[ i ].r := Boolean( ALStorageReadOnly( storage ) );
      z^[ i ].a := Boolean( ALStorageArchive( storage ) );
      z^[ i ].s := Boolean( ALStorageSystem( storage ) );
      z^[ i ].h := Boolean( ALStorageHidden( storage ) );
      z^[ i ].d := Boolean( ALStorageDirectory( storage ) );
      ALStorageGetDate( storage, z^[i].month, z^[i].date, z^[i].year );
      ALStorageGetTime( storage, z^[i].hour, z^[i].minute, z^[i].second );
      d := ALEntryGetDecompressor( entry );
      if ( d = nil) or ( ALDecompressorGetTypeCode( d ) = AL_COMPRESSION_COPY ) then
        z^[i].level := 0
      else
      begin
        z^[i].level := Byte( ALPkDecompressorLevel( d ) );
      end;
      entry := ALEntryGetNextEntry( entry );
    end;
    z^[ count ] := ALZipDirEntry.Create;
    z^[ count ].name := '';
    z^[ count ].comment := ALArchiveGetComment( arc );
    z^[ count ].size := -1;
    z^[ count ].compressed_size := LongInt( arc );
    ALReadDir := z;
  end;
  if list <> nil then deleteALEntryList( list );
end;

{*
 * NAME
 *
 *  ALFreeDir()
 *
 * PLATFORMS/ENVIRONMENTS
 *
 *  Windows
 *  Delphi
 *
 * SHORT DESCRIPTION
 *
 *  The simplified interface function frees the memory allocated for
 *  an ALZipDir.
 *
 * DELPHI SYNOPSIS
 *
 *  procedure ALFreeDir( z : ALZipDir );
 *
 * ARGUMENTS
 *
 *  z             :  The ALZipDir array.  This is the array you will have
 *                   have read in using ALReadDir().
 *
 * DESCRIPTION
 *
 *  Any time you read in the directory from a PKZIP file using ALReadDir(),
 *  you must eventually delete it using this function.  It takes care of
 *  freeing up the space used by the array itself, by the file names, the
 *  file comments, the archive comment, and the archive object itself.
 *
 * RETURNS
 *
 *  Nothing.
 *
 * EXAMPLE
 *
 * SEE ALSO
 *
 * REVISION HISTORY
 *
 *   February 14, 1996  2.0A : New Release
 *}

procedure ALFreeDir( z : ALZipDir );
var
  i : integer;
  arc : hALArchive;
begin
  i := 0;
  while z^[ i ].size <> -1 do
  begin
    z^[i].Destroy;
    i := i + 1;
  end;
  arc := hALArchive( z^[ i ].compressed_size );
  z^[i].Destroy;
  deleteALArchive( arc );
  FreeMem( z, sizeof( ALZipDirEntry ) * ( i + 1 ) );
end;

{*
 * NAME
 *
 *  ALWriteDir()
 *
 * PLATFORMS/ENVIRONMENTS
 *
 *  Windows
 *  Delphi
 *
 * SHORT DESCRIPTION
 *
 *  This function writes an ALZipDir array out to a ZIP file.
 *
 * DELPHI SYNOPSIS
 *
 *  function ALWriteDir( z : ALZipDir ) : integer;
 *
 * ARGUMENTS
 *
 *  z             :  A pointer to an ALZipDir array.  The contents of
 *                   this array are going to be written out to the
 *                   ZIP file, completely replacing its present directory.
 *
 * DESCRIPTION
 *
 *  This function writes a new directory out to a ZIP file.  This is
 *  a reasonable thing to do if read in the directory from the same
 *  ZIP file, and have udpated some file names, comments, permission
 *  bits, etc.
 *
 * RETURNS
 *
 *  A standard ArchiveLib return, AL_SUCCESS if things went well, and
 *  something < 0 if things went bad.
 *
 * EXAMPLE
 *
 * SEE ALSO
 *
 * REVISION HISTORY
 *
 *   February 14, 1996  2.0A : New Release
 *}

function ALWriteDir( z : ALZipDir ) : integer;
var
  count : integer;
  arc : hALArchive;
  list : hALEntryList;
  i : integer;
begin
  count := 0;
  while z^[ count ].size <> -1 do count := count + 1;
  arc := hALArchive( z^[ count ].compressed_size );
  list := newALListPkDecompressFileTools( nil );
  for i := 0 to count - 1 do
    ALWriteDirEntryDelphi( z^[ i ], list );
  ALArchiveSetComment( arc, z^[ count ].comment );
  ALWriteDir := ALArchiveWriteDirectory( arc, list );
end;

{*
 * NAME
 *
 *  ALZipDirEntry.get_name
 *
 * PLATFORMS/ENVIRONMENTS
 *
 *  Windows
 *  Delphi
 *
 * SHORT DESCRIPTION
 *
 *  This provides access to the name member of an ALZipDir entry.
 *
 * DELPHI SYNOPSIS
 *
 *  function ALZipDirEntry.get_name : string;
 *
 * ARGUMENTS
 *
 *  None.
 *
 * DESCRIPTION
 *
 *  This function reads the name member out of an ALZipDir entry.  It
 *  provides the service of creating a new string, instead of
 *  making Pascal users deal with the PChar member.
 *
 * RETURNS
 *
 *  A string containing the name of the file in the entry.
 *
 * EXAMPLE
 *
 * SEE ALSO
 *
 * REVISION HISTORY
 *
 *   February 14, 1996  2.0A : New Release
 *}

function ALZipDirEntry.get_name : string;
var
  s : string;
begin
  get_name := StrPas( pname );
end;

{*
 * NAME
 *
 *  ALZipDirEntry.get_comment
 *
 * PLATFORMS/ENVIRONMENTS
 *
 *  Windows
 *  Delphi
 *
 * SHORT DESCRIPTION
 *
 *  This provides access to the comment member of an ALZipDir entry.
 *
 * DELPHI SYNOPSIS
 *
 *  function ALZipDirEntry.get_comment : string;
 *
 * ARGUMENTS
 *
 *  None.
 *
 * DESCRIPTION
 *
 *  This function reads the comment member out of an ALZipDir entry.  It
 *  provides the service of creating a new string, instead of
 *  making Pascal users deal with the PChar member.
 *
 * RETURNS
 *
 *  A string containing the comment for the file in the entry.
 *
 * EXAMPLE
 *
 * SEE ALSO
 *
 * REVISION HISTORY
 *
 *   February 14, 1996  2.0A : New Release
 *}

function ALZipDirEntry.get_comment : string;
var
  s : string;
begin
  get_comment := StrPas( pcomment );
end;

{*
 * NAME
 *
 *  ALZipDirEntry.put_name
 *
 * PLATFORMS/ENVIRONMENTS
 *
 *  Windows
 *  Delphi
 *
 * SHORT DESCRIPTION
 *
 *  This provides access to the name member of an ALZipDir entry.
 *
 * DELPHI SYNOPSIS
 *
 *  procedure ALZipDirEntry.put_name( x :  string );
 *
 * ARGUMENTS
 *
 *  x  : The string that is going to be stored in the file name member.
 *
 * DESCRIPTION
 *
 *  This function writes a new name member into an ALZipDir entry.  It
 *  provides the service of creating a new PCHar, instead of
 *  making Pascal users deal with that allocation BS.
 *
 * RETURNS
 *
 *  Nothing.
 *
 * EXAMPLE
 *
 * SEE ALSO
 *
 * REVISION HISTORY
 *
 *   February 14, 1996  2.0A : New Release
 *}

procedure ALZipDirEntry.put_name( x : string );
begin
  if pname <> nil Then
    StrDispose( pname );
  pname := Stralloc( Length( x ) + 1 );
  strpcopy( pname, x );
end;

{*
 * NAME
 *
 *  ALZipDirEntry.put_comment
 *
 * PLATFORMS/ENVIRONMENTS
 *
 *  Windows
 *  Delphi
 *
 * SHORT DESCRIPTION
 *
 *  This provides access to the comment member of an ALZipDir entry.
 *
 * DELPHI SYNOPSIS
 *
 *  procedure ALZipDirEntry.put_comment( x :  string );
 *
 * ARGUMENTS
 *
 *  x  : The string that is going to be stored in the comment member.
 *
 * DESCRIPTION
 *
 *  This function writes a new comment member into an ALZipDir entry.  It
 *  provides the service of creating a new PChar, instead of
 *  making Pascal users deal with that allocation stuff.
 *
 * RETURNS
 *
 *  Nothing.
 *
 * EXAMPLE
 *
 * SEE ALSO
 *
 * REVISION HISTORY
 *
 *   February 14, 1996  2.0A : New Release
 *}

procedure ALZipDirEntry.put_comment( x : string );
begin
  if pcomment <> nil Then
    StrDispose( pcomment );
  pcomment := Stralloc( Length( x ) + 1 );
  strpcopy( pcomment, x );
end;

{*
 * NAME
 *
 *  ALZipDirEntry.Create
 *
 * PLATFORMS/ENVIRONMENTS
 *
 *  Windows
 *  Delphi
 *
 * SHORT DESCRIPTION
 *
 *  The constructor for an ALZipDirEntry object.
 *
 * DELPHI SYNOPSIS
 *
 *  constructor ALZipDirEntry.Create;
 *
 * ARGUMENTS
 *
 *  None.
 *
 * DESCRIPTION
 *
 *  Each ALZipDirEntry object represents one file in a ZIP file.
 *  When I first create one of these guys, I need to init the name
 *  and comment members to nil, so I don't try to free up garbage
 *  memory pointers later on.
 *
 * RETURNS
 *
 *  Nothing.
 *
 * EXAMPLE
 *
 * SEE ALSO
 *
 * REVISION HISTORY
 *
 *   February 14, 1996  2.0A : New Release
 *}

constructor ALZipDirEntry.Create;
begin
  pname := nil;
  pcomment := nil;
end;

{*
 * NAME
 *
 *  ALZipDirEntry.Destroy
 *
 * PLATFORMS/ENVIRONMENTS
 *
 *  Windows
 *  Delphi
 *
 * SHORT DESCRIPTION
 *
 *  The destructor for an ALZipDirEntry object.
 *
 * DELPHI SYNOPSIS
 *
 *  destructor ALZipDirEntry.Destroy;
 *
 * ARGUMENTS
 *
 *  None.
 *
 * DESCRIPTION
 *
 *  Each ALZipDirEntry object represents one file in a ZIP file.
 *  When I destroy one of these guys, I need to free the memory that
 *  was allocated to hold the name and comment members.
 *
 * RETURNS
 *
 *  Nothing.
 *
 * EXAMPLE
 *
 * SEE ALSO
 *
 * REVISION HISTORY
 *
 *   February 14, 1996  2.0A : New Release
 *}

destructor ALZipDirEntry.Destroy;
begin
  if pname <> nil then StrDispose( pname );
  if pcomment <> nil then StrDispose( pcomment );
end;

{*
 * DELPHI WRAPPER FUNCTION
 *
 * NAME
 *
 *  ALCreate()
 *
 *  This wrapper function simply performs translation between C-Style
 *  PChar data and Pascal-style Strings.  For complete documentation
 *  on what this function does, see the source for the C/VB function
 *  of the same name in SIMPCREA.CPP
 *
 * REVISION HISTORY
 *
 *  February 14, 1996  2.0A : First release
 *}

function ALCreate( archive_name : String;
                   input_files : String;
                   strip_path : integer;
                   text_window : Hwnd;
                   file_progress_window : Hwnd;
                   job_progress_window : Hwnd ) : integer;
var
  p1 : PChar;
  p2 : PChar;
begin
  p1 := StrAlloc( Length( archive_name ) + 1 );
  StrPCopy( p1, archive_name );
  p2 := StrAlloc( Length( input_files ) + 1 );
  StrPCopy( p2, input_files );
  ALCreate := ALCreatePChar( p1,
                             p2,
                             strip_path,
                             text_window,
                             file_progress_window,
                             job_progress_window );
  StrDispose( p1 );
  StrDispose( p2 );
end;

{*
 * DELPHI WRAPPER FUNCTION
 *
 * NAME
 *
 *  ALAppend()
 *
 *  This wrapper function simply performs translation between C-Style
 *  PChar data and Pascal-style Strings.  For complete documentation
 *  on what this function does, see the source for the C/VB function
 *  of the same name in SIMPAPP.CPP
 *
 * REVISION HISTORY
 *
 *  February 14, 1996  2.0A : First release
 *}

function ALAppend( archive_name : String;
                   input_files : String;
                   strip_path : integer;
                   text_window : Hwnd;
                   file_progress_window : Hwnd;
                   job_progress_window : Hwnd ) : integer;
var
  p1 : PChar;
  p2 : PChar;
begin
  p1 := StrAlloc( Length( archive_name ) + 1 );
  StrPCopy( p1, archive_name );
  p2 := StrAlloc( Length( input_files ) + 1 );
  StrPCopy( p2, input_files );
  ALAppend := ALAppendPChar( p1,
                             p2,
                             strip_path,
                             text_window,
                             file_progress_window,
                             job_progress_window );
  StrDispose( p1 );
  StrDispose( p2 );
end;

{*
 * NAME
 *
 *  ALDelete()
 *
 * PLATFORMS/ENVIRONMENTS
 *
 *  Windows
 *  Delphi
 *
 * SHORT DESCRIPTION
 *
 *  The simplified interface function to delete files from a ZIP file.
 *
 * DELPHI SYNOPSIS
 *
 *  function ALDelete( z : ALZipDir;
 *                     text_window : Hwnd;
 *                     file_progress_window : Hwnd;
 *                     job_progress_window : Hwnd ) : integer;
 *
 * ARGUMENTS
 *
 *  z             :  The ALZipDir array.  This is the array you will have
 *                   have read in using ALReadDir().  Every entry in the
 *                   array that has a mark set will be deleted.
 *
 *  text_window          : The handle for the window that will receive
 *                         file names as they are processed.
 *
 *  file_progress_window : The handle for the window that will receive
 *                         updates on the percentage of each file that
 *                         has been processed.
 *
 *  job_progress_window  : The handle for the window that will receive
 *                         updates on the percentage of the entire job that
 *                         has been processed.
 *
 * DESCRIPTION
 *
 *   The simplified ALDelete function deletes the files you specified
 *   in the ALZipDir array.  To do this, the function has to recover
 *   the ALArchive pointer that is stashed in the last element of the
 *   array.  Then, it builds an ALEntryList that is used to call
 *   ALArchive::Delete().
 *
 *   Note that the monitor works kind of funky in here.  Instead of displayin
 *   the names of the files being deleted, it shows the names of the files
 *   that *aren't* being deleted.
 *
 * RETURNS
 *
 *  AL_SUCCESS if things went well, o/w an ArchiveLib error code.
 *
 * EXAMPLE
 *
 * SEE ALSO
 *
 * REVISION HISTORY
 *
 *   February 14, 1996  2.0A : New Release
 *
 *   April 2, 1996      2.01A : Modified to take into account the fact
 *                              that ALArchiveDelete returns a file count,
 *                              not AL_SUCCESS, on success.
 *}

function ALDelete( z : ALZipDir;
                   text_window : Hwnd;
                   file_progress_window : Hwnd;
                   job_progress_window : Hwnd ) : integer;
var
  count : integer;
  arc : hALArchive;
  list : hALEntryList;
  i : integer;
  hOutput : hALArchive;
  hMonitor : hALMonitor;

begin
  count := 0;
  while z^[ count ].size <> -1 do count := count + 1;
  arc := hALArchive( z^[ count ].compressed_size );
  hMonitor := newALSimpleMonitor( text_window, file_progress_window, job_progress_window );
  list := newALListPkDecompressFileTools( hMonitor );
  for i := 0 to count - 1 do
    ALWriteDirEntryDelphi( z^[ i ], list );
  ALArchiveSetComment( arc, z^[ count ].comment );
  hOutput := newALPKArchive( '' );
  i := ALArchiveDelete( arc, list, hOutput );
  if i >= 0 then
    ALDelete := AL_SUCCESS
  else
    ALDelete := i;
  deleteALEntryList( list );
  deleteALArchive( hOutput );
  deleteALMonitor( hMonitor );
end;

{*
 * NAME
 *
 *  ALExtract()
 *
 * PLATFORMS/ENVIRONMENTS
 *
 *  Windows
 *  Delphi
 *
 * SHORT DESCRIPTION
 *
 *  The simplified interface function to extract files from a ZIP file.
 *
 * DELPHI SYNOPSIS
 *
 *  function ALExtract( z : ALZipDir;
 *                      strip_path : integer;
 *                      text_window : Hwnd;
 *                      file_progress_window : Hwnd;
 *                      job_progress_window : Hwnd ) : integer;
 *
 * ARGUMENTS
 *
 *  z             :  The ALZipDir array.  This is the array you will have
 *                   have read in using ALReadDir().  Every entry in the
 *                   array that has a mark set will be extracted.
 *
 *  strip_path    :  If this flag is set, the files that are extracted
 *                   from the archive will have their paths stripped
 *                   before the extraction takes place.
 *
 *  text_window          : The handle for the window that will receive
 *                         file names as they are processed.
 *
 *  file_progress_window : The handle for the window that will receive
 *                         updates on the percentage of each file that
 *                         has been processed.
 *
 *  job_progress_window  : The handle for the window that will receive
 *                         updates on the percentage of the entire job that
 *                         has been processed.
 *
 * DESCRIPTION
 *
 *   The simplified ALExtract function extracts the files you specified
 *   in the ALZipDir array.  To do this, the function has to recover
 *   the ALArchive pointer that is stashed in the last element of the
 *   array.  Then, it builds an ALEntryList that is used to call
 *   ALArchive::Extract().
 *
 * RETURNS
 *
 *  AL_SUCCESS if things went well, o/w an ArchiveLib error code.
 *
 * EXAMPLE
 *
 * SEE ALSO
 *
 * REVISION HISTORY
 *
 *   February 14, 1996  2.0A : New Release
 *
 *   April 2, 1996      2.01A : Modified to take into account the fact
 *                              that ALArchiveDelete returns a file count,
 *                              not AL_SUCCESS, on success.
 *}

function ALExtract( z : ALZipDir;
                    strip_path : integer;
                    text_window : Hwnd;
                    file_progress_window : Hwnd;
                    job_progress_window : Hwnd ) : integer;
var
  count : integer;
  arc : hALArchive;
  list : hALEntryList;
  i : integer;
  hMonitor : hALMonitor;
begin
  count := 0;
  while z^[ count ].size <> -1 do count := count + 1;
  arc := hALArchive( z^[ count ].compressed_size );
  hMonitor := newALSimpleMonitor( text_window, file_progress_window, job_progress_window );
  list := newALListPkDecompressFileTools( hMonitor );
  for i := 0 to count - 1 do
    ALWriteDirEntryDelphi( z^[ i ], list );
  ALArchiveSetComment( arc, z^[ count ].comment );
  ALArchiveSetStripOnExtract( arc, strip_path );
  i := ALArchiveExtract( arc, list );
  if i >= 0 then
    ALExtract := AL_SUCCESS
  else
    ALExtract := i;
  deleteALEntryList( list );
  deleteALMonitor( hMonitor );
end;

end.
